@@ -1,7 +1,7 @@
 /* common.c - common functions used by daisy-player and eBook-speaker.
  *
  * Copyright (C)2018 J. Lemmens
- *
+ *                                                                    
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
  * Free Software Foundation; either version 2 of the License, or (at your
@@ -19,7 +19,50 @@
 
 #include "daisy.h"
 
-char sink_info[10][100];                           
+char sink_info[10][100];
+
+void open_xml_file (misc_t *misc, my_attribute_t *my_attribute,
+                    daisy_t *daisy, char *xml_file, char *anchor)
+{
+   if (access (xml_file, R_OK))
+   {
+      endwin ();
+      printf ("%s\n",
+              gettext ("Please try to play this book with daisy-player"));
+      beep ();
+      quit_daisy_player (misc, my_attribute, daisy);
+      _exit (EXIT_SUCCESS);
+   } // if
+   if (! (misc->doc = htmlParseFile (xml_file, "UTF-8")))
+   {
+      int e;
+      char str[MAX_STR];
+
+      e = errno;
+      snprintf (str, MAX_STR, "open_xml_file (%s)", xml_file);
+      failure (misc, str, e);
+   } // if
+   if ((misc->reader = xmlReaderWalker (misc->doc)) == NULL)
+   {
+      int e;
+      char str[MAX_STR];
+
+      e = errno;
+      snprintf (str, MAX_STR, "open_xml_file (%s)", xml_file);
+      failure (misc, str, e);
+   } // if
+   if (! *anchor)
+      return;
+
+// skip to anchor
+   while (1)
+   {
+      if (! get_tag_or_label (misc, my_attribute, misc->reader))
+         break;
+      if (strcasecmp (anchor, my_attribute->id) == 0)
+         break;
+   } // while
+} // open_xml_file
 
 char *convert_URL_name (misc_t *misc, char *file)
 {
@@ -56,8 +99,8 @@ void failure (misc_t *misc, char *str, int e)
    printf ("\n\n%s: %s\n", str, strerror (e));
    fflush (stdout);
    remove_tmp_dir (misc);
-   printf ("reset");
-   kill (misc->main_pid, SIGTERM);
+   switch (system ("reset"));
+   _exit (EXIT_FAILURE);
 } // failure
 
 void player_ended ()
@@ -68,7 +111,7 @@ void player_ended ()
    while ((kidpid = waitpid (-1, &status, WNOHANG)) > 0);
 } // player_ended
 
-void get_path_name (char *dir, char *search_str, char *found)
+void get_realpath_name (char *dir, char *search_str, char *found)
 {
    int total, n;
    struct dirent **namelist;
@@ -76,26 +119,29 @@ void get_path_name (char *dir, char *search_str, char *found)
    total = scandir ((const char *) dir, &namelist, NULL, alphasort);
    if (total == -1)
    {
-      printf ("scandir: %s: %s\n", dir, strerror (errno));
+      printf ("\nscandir: %s: %s\n", dir, strerror (errno));
       exit (EXIT_FAILURE);
    } // if
    for (n = 0; n < total; n++)
    {
-      char *path;
+      char *path, *basec;
 
       path = malloc (strlen (dir) + strlen (namelist[n]->d_name) + 2);
       if (dir[strlen (dir) - 1] == '/')
          sprintf (path, "%s%s", dir, namelist[n]->d_name);
       else
          sprintf (path, "%s/%s", dir, namelist[n]->d_name);
-      if (strcasecmp (namelist[n]->d_name, basename (search_str)) == 0)
+      basec = strdup (search_str);
+      if (strcasecmp (namelist[n]->d_name, basename (basec)) == 0)
       {
          strcpy (found, path);
          free (path);
          free (namelist);
+         free (basec);
          return;
       } // if
       free (path);
+      free (basec);
       if (strcmp (namelist[n]->d_name, ".") == 0 ||
           strcmp (namelist[n]->d_name, "..") == 0)
          continue;
@@ -108,12 +154,12 @@ void get_path_name (char *dir, char *search_str, char *found)
             sprintf (new_dir, "%s/%s", dir, namelist[n]->d_name);
          else
             sprintf (new_dir, "%s%s", dir, namelist[n]->d_name);
-         get_path_name (new_dir, search_str, found);
+         get_realpath_name (new_dir, search_str, found);
          free (new_dir);
       } // if
    } // for
    free (namelist);
-} // get_path_name
+} // get_realpath_name
 
 char *get_dir_content (misc_t *misc, char *dir_name, char *search_str)
 {
@@ -164,38 +210,32 @@ char *get_dir_content (misc_t *misc, char *dir_name, char *search_str)
 
 void find_index_names (misc_t *misc)
 {
+   char *dirc;
 
-   get_path_name (misc->daisy_mp, "ncc.html", misc->ncc_html);
+   get_realpath_name (misc->daisy_mp, "ncc.html", misc->ncc_html);
    *misc->ncx_name = 0;
-   strncpy (misc->ncx_name,
-            get_dir_content (misc, misc->daisy_mp, ".ncx"),
-            MAX_STR - 1);
+   strcpy (misc->ncx_name,
+            get_dir_content (misc, misc->daisy_mp, ".ncx"));
    *misc->opf_name = 0;
-   strncpy (misc->opf_name,
-            get_dir_content (misc, misc->daisy_mp, ".opf"),
-            MAX_STR - 1);
+   strcpy (misc->opf_name,
+            get_dir_content (misc, misc->daisy_mp, ".opf"));
+   dirc = strdup (misc->opf_name);
+   free (misc->daisy_mp);
+   misc->daisy_mp = strdup (dirname (dirc));
 } // find_index_names
 
 void kill_player (misc_t *misc)
 {
    while (kill (misc->player_pid, SIGHUP) == 0);
-#ifdef EBOOK_SPEAKER
-   unlink (misc->eBook_speaker_txt);
-   unlink (misc->tmp_wav);
-#endif
-#ifdef DAISY_PLAYER
    if (misc->cd_type == CDIO_DISC_MODE_CD_DA)
       while (kill (misc->cdda_pid, SIGKILL) == 0);
    unlink (misc->tmp_wav);
-#endif
 } // kill_player
 
 void skip_right (misc_t *misc, daisy_t *daisy)
 {
-#ifdef DAISY_PLAYER
    if (misc->cd_type == CDIO_DISC_MODE_CD_DA)
       return;
-#endif
    if (misc->playing == -1)
    {
       beep ();
@@ -213,10 +253,13 @@ int handle_ncc_html (misc_t *misc, my_attribute_t *my_attribute,
 // lookfor "ncc.html"
    htmlDocPtr doc;
    xmlTextReaderPtr ncc;
+   char *dirc;
 
    free (misc->daisy_mp);
    misc->daisy_mp = strdup (misc->ncc_html);
-   misc->daisy_mp = dirname (misc->daisy_mp);
+   dirc = strdup (misc->daisy_mp);
+   misc->daisy_mp = strdup (dirname (dirc));
+   free (dirc);
    strncpy (misc->daisy_version, "2.02", 4);
    doc = htmlParseFile (misc->ncc_html, "UTF-8");
    ncc = xmlReaderWalker (doc);
@@ -243,12 +286,7 @@ int handle_ncc_html (misc_t *misc, my_attribute_t *my_attribute,
       printf ("%s\n",
               gettext ("Please try to play this book with daisy-player"));
       beep ();
-#ifdef DAISY_PLAYER
       quit_daisy_player (misc, my_attribute, daisy);
-#endif
-#ifdef EBOOK_SPEAKER
-      quit_eBook_speaker (misc, my_attribute, daisy);
-#endif
       _exit (EXIT_SUCCESS);
    } // if
    return misc->total_items;
@@ -368,6 +406,11 @@ daisy_t *create_daisy_struct (misc_t *misc,
 // handle "ncc.html"
    if (*misc->ncc_html)
    {
+      if (misc->verbose)
+      {
+         printf ("\r\n\ncount items in %s... ", misc->ncc_html);
+         fflush (stdout);
+      } // if
       misc->total_items = handle_ncc_html (misc, my_attribute, daisy);
       return calloc (misc->total_items + 1, sizeof (daisy_t));
    } // if ncc.html
@@ -392,6 +435,11 @@ daisy_t *create_daisy_struct (misc_t *misc,
    } // if
 
 // count items in opf
+   if (misc->verbose)
+   {
+      printf ("\r\n\ncount items in OPF... ");
+      fflush (stdout);
+   } // if
    misc->items_in_opf = 0;
    doc = htmlParseFile (misc->opf_name, "UTF-8");
    ptr = xmlReaderWalker (doc);
@@ -400,12 +448,21 @@ daisy_t *create_daisy_struct (misc_t *misc,
       if (! get_tag_or_label (misc, my_attribute, ptr))
          break;
       if (strcasecmp (misc->tag, "itemref") == 0)
-         misc->items_in_opf++;
+      {
+         misc->items_in_opf++; //
+      } // if
    } // while
    xmlTextReaderClose (ptr);
-   xmlFreeDoc (doc);
+   xmlFreeDoc (doc);                             
+   if (misc->verbose)
+      printf ("%d\n", misc->items_in_opf);
 
 // count items in ncx
+   if (misc->verbose)
+   {
+      printf ("\rcount items in NCX... ");
+      fflush (stdout);
+   } // if
    misc->items_in_ncx = 0;
    doc = htmlParseFile (misc->ncx_name, "UTF-8");
    ptr = xmlReaderWalker (doc);
@@ -418,6 +475,8 @@ daisy_t *create_daisy_struct (misc_t *misc,
    } // while
    xmlTextReaderClose (ptr);
    xmlFreeDoc (doc);
+   if (misc->verbose)
+      printf ("%d\n", misc->items_in_ncx);
 
    if (misc->items_in_ncx == 0 && misc->items_in_opf == 0)
    {
@@ -425,12 +484,7 @@ daisy_t *create_daisy_struct (misc_t *misc,
       printf ("%s\n",
               gettext ("Please try to play this book with daisy-player"));
       beep ();
-#ifdef DAISY_PLAYER
       quit_daisy_player (misc, my_attribute, daisy);
-#endif
-#ifdef EBOOK_SPEAKER
-      quit_eBook_speaker (misc, my_attribute, daisy);
-#endif
       _exit (EXIT_SUCCESS);
    } // if
 
@@ -438,12 +492,6 @@ daisy_t *create_daisy_struct (misc_t *misc,
    if (misc->items_in_opf > misc->items_in_ncx)
       misc->total_items = misc->items_in_opf;
    switch (chdir (misc->daisy_mp));
-#ifdef EBOOK_SPEAKER
-   snprintf (misc->eBook_speaker_txt, MAX_STR,
-             "%s/eBook-speaker.txt", misc->daisy_mp);
-   snprintf (misc->tmp_wav, MAX_STR,
-             "%s/eBook-speaker.wav", misc->daisy_mp);
-#endif
    return (daisy_t *) calloc (misc->total_items + 1, sizeof (daisy_t));
 } // create_daisy_struct
 
@@ -459,15 +507,6 @@ void make_tmp_dir (misc_t *misc)
       beep ();
       failure (misc, misc->tmp_dir, e);
    } // if
-#ifdef EBOOK_SPEAKER
-   switch (chdir (misc->tmp_dir))
-   {
-   case 01:
-      failure (misc, misc->tmp_dir, errno);
-   default:
-      break;
-   } // switch
-#endif
 } // make_tmp_dir
 
 void remove_tmp_dir (misc_t *misc)
@@ -478,13 +517,11 @@ void remove_tmp_dir (misc_t *misc)
       return;
 
 // Be sure not to remove wrong files
-#ifdef DAISY_PLAYER
    if (misc->cd_type == CDIO_DISC_MODE_CD_DA)
    {
       rmdir (misc->tmp_dir);
       return;
    } // if
-#endif
 
    snprintf (misc->cmd, MAX_CMD - 1, "rm -rf %s", misc->tmp_dir);
    if (system (misc->cmd) != 0)
@@ -502,31 +539,12 @@ void remove_tmp_dir (misc_t *misc)
 void get_attributes (misc_t *misc, my_attribute_t *my_attribute,
                      xmlTextReaderPtr ptr)
 {
-   char attr[MAX_STR + 1];
+   char attr[MAX_STR - 1];
 
    snprintf (attr, MAX_STR - 1, "%s", (char *)
              xmlTextReaderGetAttribute (ptr, BAD_CAST "class"));
    if (strcmp (attr, "(null)"))
-      snprintf (my_attribute->class, MAX_STR - 1, "%s", attr);
-#ifdef EBOOK_SPEAKER
-   if (misc->option_b == 0)
-   {
-      snprintf (attr, MAX_STR - 1, "%s", (char *)
-                xmlTextReaderGetAttribute
-                               (ptr, (xmlChar *) "break_on_eol"));
-      if (strcmp (attr, "(null)"))
-      {
-         misc->break_phrase = attr[0];
-         if (misc->break_phrase != 'y' && misc->break_phrase != 'n')
-            misc->break_phrase = 'n';
-      } // if
-   } // if
-   snprintf (attr, MAX_STR - 1, "%s", (char *)
-             xmlTextReaderGetAttribute (ptr, BAD_CAST "my_class"));
-   if (strcmp (attr, "(null)"))
-      strncpy (my_attribute->my_class, attr, MAX_STR - 1);
-#endif
-#ifdef DAISY_PLAYER
+      sprintf (my_attribute->class, "%s", attr);
    snprintf (attr, MAX_STR - 1, "%s", (char *)
              xmlTextReaderGetAttribute (ptr, BAD_CAST "clip-begin"));
    if (strcmp (attr, "(null)"))
@@ -543,7 +561,6 @@ void get_attributes (misc_t *misc, my_attribute_t *my_attribute,
              xmlTextReaderGetAttribute (ptr, (const xmlChar *) "clipend"));
    if (strcmp (attr, "(null)"))
       strncpy (my_attribute->clip_end, attr, MAX_STR - 1);
-#endif
    snprintf (attr, MAX_STR - 1, "%s", (char *)
              xmlTextReaderGetAttribute (ptr, (const xmlChar *) "href"));
    if (strcmp (attr, "(null)"))
@@ -553,13 +570,11 @@ void get_attributes (misc_t *misc, my_attribute_t *my_attribute,
       free (my_attribute->id);
       my_attribute->id = strdup ((char *) xmlTextReaderGetAttribute (ptr,
                (const xmlChar *) "id"));
-#ifdef DAISY_PLAYER
       if (strcmp (my_attribute->id, misc->current_id) != 0)
       {
          free (misc->current_id);
          misc->current_id = strdup (my_attribute->id);
-      } // if   
-#endif
+      } // if
    } // if
    if (xmlTextReaderGetAttribute (ptr, (const xmlChar *) "idref") != NULL)
    {
@@ -632,13 +647,6 @@ void get_attributes (misc_t *misc, my_attribute_t *my_attribute,
            xmlTextReaderGetAttribute (ptr, (const xmlChar *) "playorder"));
    if (strcmp (attr, "(null)"))
       strncpy (my_attribute->playorder, attr, MAX_STR - 1);
-#ifdef EBOOK_SPEAKER
-   snprintf (attr, MAX_STR - 1, "%s", (char *)
-             xmlTextReaderGetAttribute (ptr, (const xmlChar *) "phrase"));
-   if (strcmp (attr, "(null)"))
-      misc->phrase_nr = atoi ((char *) attr);
-#endif
-#ifdef DAISY_PLAYER
    snprintf (attr, MAX_STR - 1, "%s", (char *)
              xmlTextReaderGetAttribute (ptr, (const xmlChar *) "seconds"));
    if (strcmp (attr, "(null)"))
@@ -647,7 +655,6 @@ void get_attributes (misc_t *misc, my_attribute_t *my_attribute,
       if (misc->seconds < 0)
          misc->seconds = 0;
    } // if
-#endif
    snprintf (attr, MAX_STR - 1, "%s", (char *)
              xmlTextReaderGetAttribute (ptr, (const xmlChar *) "smilref"));
    if (strcmp (attr, "(null)"))
@@ -665,12 +672,10 @@ void get_attributes (misc_t *misc, my_attribute_t *my_attribute,
              xmlTextReaderGetAttribute (ptr, (const xmlChar *) "cd_dev"));
    if (strcmp (attr, "(null)"))
       strncpy (misc->cd_dev, attr, MAX_STR - 1);
-#ifdef DAISY_PLAYER
    snprintf (attr, MAX_STR - 1, "%s", (char *)
            xmlTextReaderGetAttribute (ptr, (const xmlChar *) "cddb_flag"));
    if (strcmp (attr, "(null)"))
       misc->cddb_flag = (char) attr[0];
-#endif
    snprintf (attr, MAX_STR - 1, "%s", (char *)
              xmlTextReaderGetAttribute (ptr, (const xmlChar *) "speed"));
    if (strcmp (attr, "(null)"))
@@ -685,10 +690,6 @@ void get_attributes (misc_t *misc, my_attribute_t *my_attribute,
       my_attribute->src = strdup ((char *) xmlTextReaderGetAttribute (ptr,
                (const xmlChar *) "src"));
    } // if
-   snprintf (attr, MAX_STR - 1, "%s", (char *)
-             xmlTextReaderGetAttribute (ptr, (const xmlChar *) "tts"));
-   if (strcmp (attr, "(null)"))
-      misc->tts_no = atof ((char *) attr);
    snprintf (attr, MAX_STR - 1, "%s", (char *)
              xmlTextReaderGetAttribute (ptr, (const xmlChar *) "toc"));
    if (strcmp (attr, "(null)"))
@@ -707,13 +708,8 @@ int get_tag_or_label (misc_t *misc, my_attribute_t *my_attribute,
    *misc->tag = 0;
    misc->label = realloc (misc->label, 1);
    *misc->label = 0;
-#ifdef EBOOK_SPEAKER
-   *my_attribute->my_class = 0;
-#endif
    *my_attribute->class = 0;
-#ifdef DAISY_PLAYER
    *my_attribute->clip_begin = *my_attribute->clip_end = 0;
-#endif
    *my_attribute->href = *my_attribute->media_type =
    *my_attribute->playorder = *my_attribute->smilref = 0;
    *my_attribute->toc = *my_attribute->value = 0;
@@ -765,7 +761,6 @@ int get_tag_or_label (misc_t *misc, my_attribute_t *my_attribute,
       n = xmlTextReaderAttributeCount (reader);
       for (i = 0; i < n; i++)
          get_attributes (misc, my_attribute, reader);
-#ifdef DAISY_PLAYER
       if (strcasecmp (misc->tag, "audio") == 0)
       {
          free (misc->prev_id);
@@ -773,7 +768,6 @@ int get_tag_or_label (misc_t *misc, my_attribute_t *my_attribute,
          free (misc->audio_id);
          misc->audio_id = strdup (misc->current_id);
       } // if
-#endif
       return 1;
    case XML_READER_TYPE_END_ELEMENT:
       snprintf (misc->tag, MAX_TAG - 1, "/%s",
@@ -822,15 +816,10 @@ void go_to_page_number (misc_t *misc, my_attribute_t *my_attribute,
    char pn[15];
 
    kill_player (misc);
-#ifdef DAISY_PLAYER
    if (misc->cd_type != CDIO_DISC_MODE_CD_DA)
       misc->player_pid = -2;
-#endif
    misc->playing = misc->just_this_item = -1;
    view_screen (misc, daisy);
-#ifdef EBOOK_SPEAKER
-   remove (misc->tmp_wav);
-#endif             
    unlink ("old.wav");
    mvwprintw (misc->titlewin, 1, 0,
               "----------------------------------------");
@@ -857,47 +846,48 @@ void go_to_page_number (misc_t *misc, my_attribute_t *my_attribute,
    wattroff (misc->titlewin, A_BOLD);
    wrefresh (misc->titlewin);
    misc->playing = misc->current_page_number = 0;
-#ifdef EBOOK_SPEAKER
-   misc->phrase_nr = 0;
-#endif
+   for (misc->current = 0; misc->current < misc->total_items; misc->current++)
+   {
+      if (daisy[misc->current].page_number == atoi (pn))
+      {
+// at the start of an item
+         misc->displaying = misc->playing = misc->current;
+         open_clips_file (misc, my_attribute, daisy[misc->playing].clips_file,
+                          daisy[misc->playing].clips_anchor);
+         misc->elapsed_seconds = time (NULL);
+         return;
+      } // if
+   } // for
 
-#ifdef DAISY_PLAYER
+   for (misc->playing = misc->total_items - 1; misc->playing >= 0;
+        misc->playing--)
+   {
+      if (daisy[misc->playing].page_number < atoi (pn))
+         break;
+   } // for
+
+// start searching here
    open_clips_file (misc, my_attribute, daisy[misc->playing].clips_file,
                     daisy[misc->playing].clips_anchor);
-#endif
-#ifdef EBOOK_SPEAKER
-   open_xml_file (misc, my_attribute, daisy[misc->playing].xml_file,
-                   daisy[misc->playing].anchor);
-#endif
    while (1)
    {
       if (misc->current_page_number == atoi (pn))
       {
+// found it
          misc->displaying = misc->current = misc->playing;
          misc->player_pid = -2;
          view_screen (misc, daisy);
          return;
-      } // if (misc->current_page_number == atoi (pn))
+      } // if current_page_number == pn
       if (misc->current_page_number > atoi (pn))
       {
          misc->current = misc->playing = 0;
          misc->current_page_number = daisy[misc->playing].page_number;
-#ifdef DAISY_PLAYER
          open_clips_file (misc, my_attribute, daisy[misc->playing].clips_file,
                           daisy[misc->playing].clips_anchor);
-#endif
-#ifdef EBOOK_SPEAKER
-         open_xml_file (misc, my_attribute, daisy[misc->playing].xml_file,
-                         daisy[misc->playing].anchor);
-#endif
          return;
-      } // if
-#ifdef DAISY_PLAYER
+      } // if current_page_number >  pn
       get_next_clips (misc, my_attribute, daisy);
-#endif
-#ifdef EBOOK_SPEAKER
-      get_next_phrase (misc, my_attribute, daisy, 0);
-#endif
    } // while
 } // go_to_page_number
 
@@ -929,28 +919,20 @@ void select_next_output_device (misc_t *misc, daisy_t *daisy)
 
    wclear (misc->screenwin);
    wprintw (misc->screenwin, "\n%s\n\n", gettext ("Select a soundcard:"));
-   if (access ("/usr/bin/pacmd", X_OK) == -1)
-   {
-      int e;
-
-      e = errno;
-      printf ("\7\n\n%s: %s\n", gettext
-     ("Be sure the package pulseaudio-utils is installed onto your system."),
-              strerror (e));
-      _exit (EXIT_FAILURE);
-   } // if
 
    if (*misc->pulseaudio_device)
       c = atoi (misc->pulseaudio_device);
    else
       c = 0;
    nodelay (misc->screenwin, FALSE);
-   for (;;)
+   while (! misc->term_signaled)
    {
-      char dev[5];
+      char dev[5], *str;
+      size_t len = 0;
+      int total;
       FILE *p;
 
-      if ((p = popen ("/usr/bin/pactl list sinks", "r")) == NULL)
+      if ((p = popen ("LANG=C /usr/bin/pactl list sinks", "r")) == NULL)
       {
          beep ();
          endwin ();
@@ -959,20 +941,20 @@ void select_next_output_device (misc_t *misc, daisy_t *daisy)
          _exit (EXIT_FAILURE);
       } // if
 
-      size_t len = 0;
-      int total;
-      char *str = NULL;
-
       n = -1;
-      while (1)
+      str = NULL;
+      len = 0;
+      while (! misc->term_signaled)
       {
          if (getline (&str, &len, p) == -1)
             break;
-         if (strncmp (str, "Sink", 4) == 0)
+         if (strncasecmp (str, "Sink", 4) == 0)
          {
             memset (sink_info[++n], ' ', 70);
-            continue;
-         } // if          
+         } // if
+
+/* disabled.
+   Requested by Didier Spaier <didier@slint.fr>
          if (strstr (str, "Description:"))
          {
             int x;
@@ -982,17 +964,24 @@ void select_next_output_device (misc_t *misc, daisy_t *daisy)
                sink_info[n][x] = ' ';
             sink_info[n][x] = 0 ;
          } // if
-         if (strstr (str, "Mute:"))
+*/
+         if (strcasestr (str, "alsa.card_name"))
+         {
+            len = strlen (str + 20);
+            memcpy (sink_info[n], str + 20, len);
+            memset (sink_info[n] + len - 2, ' ', 55 - len);
+         } // if
+         if (strcasestr (str, "Mute:"))
          {
             strcpy (sink_info[n] + 53, strstr (str, "Mute:"));
             sink_info[n][52] = ' ';
          } // if
          if (strcasestr (str, "Volume:"))
          {
-            if (strcasestr (str, "Base"))
-               continue;
             int x;
 
+            if (strcasestr (str, "Base"))
+               continue;
             sink_info[n][strlen (sink_info[n]) - 1] = ' ';
             sink_info[n][strlen (sink_info[n])] = ' ';
             strcpy (sink_info[n] + 63, strstr (str, "Volume:"));
@@ -1008,6 +997,7 @@ void select_next_output_device (misc_t *misc, daisy_t *daisy)
          } // if
       } // while
       total = n + 1;
+      free (str);
       pclose (p);
 
 /* The build-in pactl function with argument list gives problems.
@@ -1079,21 +1069,24 @@ void select_next_output_device (misc_t *misc, daisy_t *daisy)
       tv.tv_usec = 100000;
 // pause till a key has been pressed or 0.1 seconds are elapsed
       select (1, &rfds, NULL, NULL, &tv);
-   } // for
+   } // while
 } // select_next_output_device
 
+// Useful to reset handlers in fork()ed children
+void reset_term_signal_handlers_after_fork (void)
+{
+   signal (SIGINT, SIG_DFL);
+   signal (SIGTERM, SIG_DFL);
+   signal (SIGHUP, SIG_DFL);
+} // reset_term_signal_handlers
+
 void free_all (misc_t *misc, my_attribute_t *my_attribute, daisy_t *daisy)
 {
-#ifdef DAISY_PLAYER
    free (misc->pause_resume_id);
    free (misc->prev_id);
    free (misc->current_id);
    free (misc->audio_id);
    free (misc->current_audio_file);
-#endif
-#ifdef EBOOK_SPEAKER
-   free (misc->src_dir);
-#endif
    free (misc->daisy_mp);
    free (misc->tmp_dir);
    free (misc->label);
@@ -1104,10 +1097,9 @@ void free_all (misc_t *misc, my_attribute_t *my_attribute, daisy_t *daisy)
         misc->current++)
    {
       free (daisy[misc->current].xml_file);
-      free (daisy[misc->current].anchor);
+      free (daisy[misc->current].xml_anchor);
       free (daisy[misc->current].clips_file);
       free (daisy[misc->current].clips_anchor);
-      free (daisy[misc->current].orig_smil);
    } // for
    delwin (misc->titlewin);
    delwin (misc->screenwin);
