@@ -1,5 +1,5 @@
 /* header file for daisy-player
- *  Copyright (C)2018 J. Lemmens
+ *  Copyright (C)2019 J. Lemmens
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -42,12 +42,8 @@
 #include <libxml/xmlwriter.h>
 #include <libxml/HTMLparser.h>
 #include <cdio/cdio.h>
-#ifdef HAVE_CDIO_PARANOIA_CDDA_H
-   #include <cdio/paranoia/paranoia.h>
-#else
-   #include <cdio/cdda.h>
-   #include <cdio/paranoia.h>
-#endif
+#include <cdda.h>
+#include <paranoia.h>
 #include <cdio/disc.h>
 #include <magic.h>
 #include <fnmatch.h>
@@ -74,8 +70,9 @@ typedef struct Daisy
 {
    int playorder, x, y, screen;
    float begin, duration;
-   char *xml_file, *anchor, *clips_file;
-   char *clips_anchor, *orig_smil;
+   char *smil_file, *smil_anchor;
+   char *xml_file, *xml_anchor;
+   char *clips_file, *clips_anchor, *orig_smil;
    char class[MAX_STR], label[100];
    char first_id[MAX_STR], last_id[MAX_STR];
    int level, page_number;
@@ -116,9 +113,10 @@ typedef struct Misc
    int discinfo, playing, just_this_item, current_page_number;
    int current, max_y, max_x, total_items, level, displaying, ignore_bookmark;
    int items_in_opf, items_in_ncx;
-   int tts_no, depth, total_pages;
+   int depth, total_pages, verbose;
    int pipefd[2], tmp_wav_fd, has_audio_tag;
    int pause_resume_playing, mounted_by_daisy_player;
+   int ncx_failed, opf_failed;
    char *pause_resume_id, *prev_id, *current_id;
    char *audio_id;
    float speed, total_time, clip_begin, clip_end;
@@ -135,7 +133,6 @@ typedef struct Misc
    char bookmark_title[MAX_STR], search_str[30];
    char cd_dev[MAX_STR], pulseaudio_device[5];
    char cddb_flag, opf_name[MAX_STR], ncx_name[MAX_STR];
-   char use_ncx, use_opf;
    char *current_audio_file, tmp_wav[MAX_STR + 1], mcn[MAX_STR];
    char xmlversion[MAX_STR + 1];
    char cmd[MAX_CMD + 1], str[MAX_STR + 1];
@@ -146,18 +143,21 @@ typedef struct Misc
    CdIo_t *p_cdio;
    int cd_type;
    lsn_t lsn_cursor, pause_resume_lsn_cursor;
+   int term_signaled;
    int use_OPF, use_NCX; // for testing
 } misc_t;
 
 extern void get_toc_audiocd (misc_t *, daisy_t *);
 extern  daisy_t *get_number_of_tracks (misc_t *);
 extern pid_t play_track (misc_t *, char *, char *, lsn_t);
-extern void playfile (char *, char *, char *, char *, char *);
+extern void playfile (misc_t *, char *, char *, char *, char *, char *);
 extern char *get_mcn (misc_t *);
 extern void quit_daisy_player (misc_t *, my_attribute_t *, daisy_t *);
 extern void view_screen (misc_t *, daisy_t *);
 extern void pause_resume (misc_t *, my_attribute_t *, daisy_t *);
 extern void open_clips_file (misc_t *, my_attribute_t *, char *, char *);
+extern void open_xml_file (misc_t *, my_attribute_t *,
+                           daisy_t *, char *, char *);
 extern void get_next_clips (misc_t *, my_attribute_t *, daisy_t *);
 extern void free_all (misc_t *, my_attribute_t *, daisy_t *);
 extern void select_next_output_device (misc_t *, daisy_t *);
@@ -167,7 +167,7 @@ extern daisy_t *create_daisy_struct (misc_t *, my_attribute_t *, daisy_t *);
 extern void skip_right (misc_t *, daisy_t *);
 extern void player_ended ();
 extern char *convert_URL_name (misc_t *, char *);
-extern void get_path_name (char *, char *, char *);
+extern void get_realpath_name (char *, char *, char *);
 extern void failure (misc_t *, char *, int);
 extern int get_tag_or_label (misc_t *, my_attribute_t *, xmlTextReaderPtr);
 extern void get_clips (misc_t *, my_attribute_t *);
@@ -193,15 +193,17 @@ extern void create_ncc_html (misc_t *);
 extern void get_attributes (misc_t *, my_attribute_t *, xmlTextReaderPtr);
 extern char *pactl (char *, char *, char *);
 extern void parse_smil_2 (misc_t *, my_attribute_t *, daisy_t *);
-extern void get_label_2 (misc_t *, daisy_t *, int);
-extern void fill_xml_anchor_ncx (misc_t *, my_attribute_t *, daisy_t *);
-extern void parse_content (misc_t *, my_attribute_t *, daisy_t *);
-extern void parse_text (misc_t *, my_attribute_t *, daisy_t *);
+extern void get_label_opf (misc_t *, my_attribute_t *, daisy_t *, int);
+extern void parse_manifest (misc_t *, my_attribute_t *, daisy_t *,
+                            int, char *);
+extern void get_label_2 (misc_t *, daisy_t *, int, int);
+extern void fill_smil_anchor_ncx (misc_t *, my_attribute_t *, daisy_t *);
+extern void parse_content_ncx (misc_t *, my_attribute_t *, daisy_t *);
 extern void put_bookmark (misc_t *);
 extern float read_time (char *);
-extern void parse_clips (misc_t *, my_attribute_t *, daisy_t *);
-extern void parse_smil_3 (misc_t *, my_attribute_t *, daisy_t *);
-extern void fill_xml_anchor_opf (misc_t *, my_attribute_t *, daisy_t *);
+extern void parse_clips_opf (misc_t *, my_attribute_t *, daisy_t *, int);
+extern void parse_smil_opf (misc_t *, my_attribute_t *, daisy_t *, int);
+extern void fill_smil_anchor_opf (misc_t *, my_attribute_t *, daisy_t *);
 extern void parse_opf (misc_t *, my_attribute_t *, daisy_t *);
 extern void parse_ncx (misc_t *, my_attribute_t *, daisy_t *);
 extern void get_bookmark (misc_t *, my_attribute_t *, daisy_t *);
@@ -224,3 +226,5 @@ extern void browse (misc_t *, my_attribute_t *, daisy_t *, char *);
 extern void usage (int);
 extern char *get_mount_point (misc_t *);
 extern void handle_discinfo (misc_t *, my_attribute_t *, daisy_t *, char *);
+extern void reset_term_signal_handlers_after_fork (void);
+extern void parse_spine (misc_t *, my_attribute_t *, daisy_t *);
